@@ -19,7 +19,7 @@ try:
 except NameError:
     basestring = str
 
-__version__ = '0.5.2'
+__version__ = '0.6.0'
 
 RE = re.compile(r"""
 (?:(?P<scheme>[^:/?#]+):)?      # scheme
@@ -27,20 +27,20 @@ RE = re.compile(r"""
 (?P<path>[^?#]*)                # path
 (?:\?(?P<query>[^#]*))?         # query
 (?:\#(?P<fragment>.*))?         # fragment
-""", flags=(re.VERBOSE))
+""", flags=re.VERBOSE)
 """Regular expression for splitting a well-formed URI into its
 components, as specified in RFC 3986 Appendix B.
 
 """
 
 GEN_DELIMS = ':/?#[]@'
-"""General delimiting characters specified in RFC 3986."""
+"""General delimiting characters as specified in RFC 3986."""
 
 SUB_DELIMS = "!$&'()*+,;="
-"""Subcomponent delimiting characters specified in RFC 3986."""
+"""Subcomponent delimiting characters as specified in RFC 3986."""
 
 RESERVED = GEN_DELIMS + SUB_DELIMS
-"""Reserved characters specified in RFC 3986."""
+"""Reserved characters as specified in RFC 3986."""
 
 UNRESERVED = (
     'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
@@ -48,34 +48,78 @@ UNRESERVED = (
     '0123456789'
     '_.-~'
 )
-"""Unreserved characters specified in RFC 3986."""
+"""Unreserved characters as specified in RFC 3986."""
 
 _URI_COMPONENTS = ('scheme', 'authority', 'path', 'query', 'fragment')
 
-_SCHEME_RE = re.compile(r"^[a-zA-Z][a-zA-Z0-9.+-]*$")
+# RFC 3986 3.2: scheme = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
+_SCHEME_RE = re.compile(r"\A[A-Z][A-Z0-9+.-]*\Z", flags=re.IGNORECASE)
 
+# RFC 3986 3.2: authority = [ userinfo "@" ] host [ ":" port ]
 _AUTHORITY_RE = re.compile(r"""
-(?:(.*)\@)?     # userinfo
+\A
+(?:(.*)@)?      # userinfo
 (.*?)           # host
-(?:\:(\d*))?    # port
-$
-""", flags=(re.VERBOSE))
+(?::(\d*))?     # port
+\Z
+""", flags=re.VERBOSE)
+
+# RFC 3986 3.2.2: A host identified by an IPv6 literal address is
+# represented inside the square brackets without a preceding version
+# flag.  The ABNF provided here is a translation of the text
+# definition of an IPv6 literal address provided in [RFC3513].  This
+# syntax does not support IPv6 scoped addressing zone identifiers.
+#
+# IPv6address =                            6( h16 ":" ) ls32
+#             /                       "::" 5( h16 ":" ) ls32
+#             / [               h16 ] "::" 4( h16 ":" ) ls32
+#             / [ *1( h16 ":" ) h16 ] "::" 3( h16 ":" ) ls32
+#             / [ *2( h16 ":" ) h16 ] "::" 2( h16 ":" ) ls32
+#             / [ *3( h16 ":" ) h16 ] "::"    h16 ":"   ls32
+#             / [ *4( h16 ":" ) h16 ] "::"              ls32
+#             / [ *5( h16 ":" ) h16 ] "::"              h16
+#             / [ *6( h16 ":" ) h16 ] "::"
+#
+# ls32        = ( h16 ":" h16 ) / IPv4address
+#             ; least-significant 32 bits of address
+#
+# h16         = 1*4HEXDIG
+#             ; 16 bits of address represented in hexadecimal
+_IPV6_ADDRESS_RE = re.compile(r"""
+\A
+(?:
+  (?:
+                                              (?:[0-9A-F]{1,4}:){6}
+  |                                         ::(?:[0-9A-F]{1,4}:){5}
+  | (?:                      [0-9A-F]{1,4})?::(?:[0-9A-F]{1,4}:){4}
+  | (?:(?:[0-9A-F]{1,4}:){,1}[0-9A-F]{1,4})?::(?:[0-9A-F]{1,4}:){3}
+  | (?:(?:[0-9A-F]{1,4}:){,2}[0-9A-F]{1,4})?::(?:[0-9A-F]{1,4}:){2}
+  | (?:(?:[0-9A-F]{1,4}:){,3}[0-9A-F]{1,4})?::   [0-9A-F]{1,4}:
+  | (?:(?:[0-9A-F]{1,4}:){,4}[0-9A-F]{1,4})?::
+  )
+  (?:
+    [0-9A-F]{1,4}:[0-9A-F]{1,4}                            # h16 ":" h16
+  | (?:(?:25[0-5]|(?:2[0-4]|1[0-9]|[1-9])?[0-9])\.?\b){4}  # IPv4address
+  )
+| (?:(?:[0-9A-F]{1,4}:){,5}[0-9A-F]{1,4})?::[0-9A-F]{1,4}
+| (?:(?:[0-9A-F]{1,4}:){,6}[0-9A-F]{1,4})?::
+)
+\Z
+""", flags=(re.IGNORECASE | re.VERBOSE))
 
 
-def _queryencode(query, delim, sep, encoding):
+def _queryencode(query, delim, encoding):
     if not delim:
         delim = ''
-    if not sep:
-        sep = ''
     # TODO: provide our own quote/unquote implementation?
-    safe = (SUB_DELIMS + ':@/?').replace(delim, '').replace(sep, '')
+    safe = (SUB_DELIMS + ':@/?').replace(delim, '')
     items = []
     for item in query:
         if isinstance(item, basestring):
             parts = (uriencode(item, safe, encoding), )
         else:
             parts = (uriencode(part, safe, encoding) for part in item)
-        items.append(sep.join(parts))
+        items.append('='.join(parts))
     return delim.join(items)
 
 
@@ -112,9 +156,9 @@ def urisplit(string):
     | :attr:`host`      |       | Host subcomponent of authority,             |
     |                   |       | or :const:`None` if not present             |
     +-------------------+-------+---------------------------------------------+
-    | :attr:`port`      |       | Port subcomponent of authority as an        |
-    |                   |       | :class:`int`, or :const:`None` if not       |
-    |                   |       | present                                     |
+    | :attr:`port`      |       | Port subcomponent of authority as a         |
+    |                   |       | (possibly empty) string,                    |
+    |                   |       | or :const:`None` if not present             |
     +-------------------+-------+---------------------------------------------+
 
     """
@@ -168,19 +212,15 @@ def uridefrag(string):
     +-------------------+-------+---------------------------------------------+
     | Attribute         | Index | Value                                       |
     +===================+=======+=============================================+
-    | :attr:`base`      | 0     | The absoulte URI or relative URI reference  |
+    | :attr:`base`      | 0     | Absoulte URI or relative URI reference      |
     |                   |       | without the fragment identifier             |
     +-------------------+-------+---------------------------------------------+
-    | :attr:`fragment`  | 1     | The fragment identifier,                    |
+    | :attr:`fragment`  | 1     | Fragment identifier,                        |
     |                   |       | or :const:`None` if not present             |
     +-------------------+-------+---------------------------------------------+
 
     """
-    try:
-        parts = string.partition('#')
-    except TypeError:
-        # Python 3: handle string as bytes
-        parts = string.partition(b'#')
+    parts = string.partition(b'#' if isinstance(string, bytes) else '#')
     if parts[1]:
         return DefragResult(parts[0], parts[2])
     else:
@@ -225,8 +265,7 @@ def urinormpath(path):
 
 
 def uricompose(scheme=None, authority=None, path='', query=None,
-               fragment=None, delim='&', querysep='=',
-               encoding='utf-8'):
+               fragment=None, delim='&', encoding='utf-8'):
     """Compose a URI string from its components.
 
     If `query` is a mapping object or a sequence of two-element
@@ -250,8 +289,39 @@ def uricompose(scheme=None, authority=None, path='', query=None,
         scheme = scheme.lower()
 
     if authority is not None:
-        # TODO: check authority is well-formed, IPv6 support
-        authority = uriencode(authority, SUB_DELIMS + ':@', encoding)
+        if isinstance(authority, basestring):
+            userinfo, host, port = _AUTHORITY_RE.match(authority).groups()
+        else:
+            userinfo, host, port = authority
+        # RFC 3986 3.2.1: The user information, if present, is
+        # followed by a commercial at-sign ("@") that delimits it from
+        # the host.
+        if userinfo is not None:
+            authority = uriencode(userinfo, SUB_DELIMS + ':', encoding) + '@'
+        else:
+            authority = ''
+        # RFC 3986 3.2.3: Although host is case-insensitive, producers
+        # and normalizers should use lowercase for registered names
+        # and hexadecimal addresses for the sake of uniformity, while
+        # only using uppercase letters for percent-encodings.
+        if host is None:
+            raise ValueError('Host subcomponent must be present if empty')
+        host = host.lower()
+        if host.startswith('[') and host.endswith(']'):
+            authority += host  # TODO: check IPv6
+        elif _IPV6_ADDRESS_RE.match(host):
+            authority += '[' + host + ']'
+        else:
+            authority += uriencode(host, SUB_DELIMS, encoding)
+        # RFC 3986 3.2.3: URI producers and normalizers should omit
+        # the port component and its ":" delimiter if port is empty or
+        # if its value would be the same as that of the scheme's
+        # default.
+        port = str(port) if port is not None else ''
+        if port.isdigit():
+            authority += ':' + port
+        elif port:
+            raise ValueError('Non-decimal port: %s' % port)
 
     # RFC 3986 3.3: If a URI contains an authority component, then the
     # path component must either be empty or begin with a slash ("/")
@@ -263,20 +333,20 @@ def uricompose(scheme=None, authority=None, path='', query=None,
     if path is None:
         raise ValueError('URI path component must be present if empty')
     if authority is not None and path and not path.startswith('/'):
-        raise ValueError('Invalid path %r with authority' % path)
+        raise ValueError('Invalid path %r with authority component' % path)
     if authority is None and path.startswith('//'):
-        raise ValueError('Invalid path %r without authority' % path)
+        raise ValueError('Invalid path %r without authority component' % path)
     if scheme is None and authority is None and ':' in path.split('/', 1)[0]:
-        raise ValueError('Invalid path %r without scheme' % path)
+        raise ValueError('Invalid path %r without scheme component' % path)
     path = uriencode(path, SUB_DELIMS + ':@/', encoding)
 
     if query is not None:
         if isinstance(query, basestring):
             query = uriencode(query, SUB_DELIMS + ':@/?', encoding)
         elif hasattr(query, 'items'):
-            query = _queryencode(query.items(), delim, querysep, encoding)
+            query = _queryencode(query.items(), delim, encoding)
         else:
-            query = _queryencode(query, delim, querysep, encoding)
+            query = _queryencode(query, delim, encoding)
 
     if fragment:
         fragment = uriencode(fragment, SUB_DELIMS + ':@/?', encoding)
@@ -289,10 +359,15 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
 
     @property
     def _splitauth(self):
-        if self.authority is None:
+        authority = self.authority
+        if authority is None:
             return (None, None, None)
-        else:
-            return _AUTHORITY_RE.match(self.authority).groups()
+        try:
+            return _AUTHORITY_RE.match(authority).groups()
+        except TypeError:
+            # Python 3: handle authority as bytes
+            groups = _AUTHORITY_RE.match(authority.decode('ascii')).groups()
+            return [None if g is None else g.encode('ascii') for g in groups]
 
     @property
     def userinfo(self):
@@ -304,8 +379,7 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
 
     @property
     def port(self):
-        port = self._splitauth[2]
-        return int(port, 10) if port else None
+        return self._splitauth[2]
 
     def geturi(self):
         """Return the re-combined version of the original URI as a string."""
@@ -313,28 +387,25 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
 
     def getscheme(self, default=None):
         """Return the URI scheme in canonical (lowercase) form, or `default`
-        if the original URI did not contain a scheme component.  Raise
-        a :class:`ValueError` if the scheme is not well-formed.
+        if the original URI did not contain a scheme component.
 
         """
         if self.scheme is None:
             return default
-        try:
-            scheme = self.scheme.decode('ascii')
-        except AttributeError:
-            scheme = self.scheme
-        if not _SCHEME_RE.match(scheme):
-            raise ValueError('Invalid scheme: %r' % scheme)
-        return scheme.lower()
+        elif isinstance(self.scheme, bytes):
+            return self.scheme.decode('ascii').lower()
+        else:
+            return self.scheme.lower()
 
     def getauthority(self, default=None, encoding='utf-8'):
         """Return the decoded URI authority, or `default` if the original URI
         did not contain an authority component.
 
         """
-        if self.authority is None:
+        if self.authority is not None:
+            return uridecode(self.authority, encoding)
+        else:
             return default
-        return uridecode(self.authority, encoding)
 
     def getpath(self, encoding='utf-8'):
         """Return the decoded URI path."""
@@ -345,18 +416,20 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
         did not contain a query component.
 
         """
-        if self.query is None:
+        if self.query is not None:
+            return uridecode(self.query, encoding)
+        else:
             return default
-        return uridecode(self.query, encoding)
 
     def getfragment(self, default=None, encoding='utf-8'):
         """Return the decoded fragment identifier, or `default` if the
         original URI did not contain a fragment component.
 
         """
-        if self.fragment is None:
+        if self.fragment is not None:
+            return uridecode(self.fragment, encoding)
+        else:
             return default
-        return uridecode(self.fragment, encoding)
 
     def getuserinfo(self, default=None, encoding='utf-8'):
         """Return the decoded userinfo subcomponent of the URI authority, or
@@ -364,9 +437,10 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
         field.
 
         """
-        if self.userinfo is None:
+        if self.userinfo is not None:
+            return uridecode(self.userinfo, encoding)
+        else:
             return default
-        return uridecode(self.userinfo, encoding)
 
     def gethost(self, default=None, encoding='utf-8'):
         """Return the decoded host subcomponent of the URI authority, or
@@ -377,17 +451,29 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
         must be specified to return a Unicode domain name.
 
         """
-        if self.host is None:
+        host = self.host
+        if isinstance(host, bytes):
+            host = host.decode(encoding)
+        # RFC 3986 3.2.2: If the URI scheme defines a default for
+        # host, then that default applies when the host subcomponent
+        # is undefined or when the registered name is empty (zero
+        # length).
+        if host is None or (default is not None and not host):
             return default
-        return uridecode(self.host, encoding)
+        elif not host.startswith('[') or not host.endswith(']'):
+            return uridecode(host, encoding)
+        elif _IPV6_ADDRESS_RE.match(host[1:-1]):
+            return host[1:-1]
+        else:
+            raise ValueError('Invalid IP literal: %s' % host)
 
     def getport(self, default=None):
         """Return the port subcomponent of the URI authority as an
         :class:`int`, or `default` if the original URI did not contain
-        a port.
+        a port, or if the port was empty.
 
         """
-        return self.port if self.port is not None else default
+        return int(self.port) if self.port else default
 
     def getaddrinfo(self, family=0, type=0, proto=0, flags=0):
         """Translate the host and port subcomponents of the URI authority into
@@ -405,42 +491,36 @@ class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
         port = self.port or self.scheme
         return socket.getaddrinfo(host, port, family, type, proto, flags)
 
-    def getquerylist(self, delims=';&', sep='=', encoding='utf-8'):
+    def getquerylist(self, delims=';&', encoding='utf-8'):
         """Split the query string into individual components using the
-        delimiter characters in `delims`.
+        delimiter characters in `delims`, and return a list of `(name,
+        value)` pairs.
 
-        If `sep` is not empty, split each component at the first
-        occurence of `sep` and return a list of decoded `(name,
-        value)` pairs.  If `sep` is not found, `value` becomes
-        :const:`None`.
-
-        If `sep` is :const:`None` or empty, return the list of decoded
-        query components.
         """
         qsl = [self.query] if self.query else []
         for delim in delims:
+            if isinstance(self.query, bytes) and isinstance(delim, str):
+                delim = delim.encode(encoding)
             qsl = [s for qs in qsl for s in qs.split(delim) if s]
-        if not sep:
-            return [uridecode(qs, encoding) for qs in qsl]
         items = []
         for qs in qsl:
-            parts = qs.partition(sep)
+            parts = qs.partition(b'=' if isinstance(qs, bytes) else '=')
             name = uridecode(parts[0], encoding)
             value = uridecode(parts[2], encoding) if parts[1] else None
             items.append((name, value))
         return items
 
-    def getquerydict(self, delims=';&', sep='=', encoding='utf-8'):
+    def getquerydict(self, delims=';&', encoding='utf-8'):
         """Split the query string into individual components using the
         delimiter characters in `delims`, and return a dictionary of
         query parameters.
 
         The dictionary keys are the unique decoded query parameter
         names, and the values are lists of decoded values for each
-        name.  Parameter names and values are seperated by `sep`.
+        name, with names and values seperated by :const:`'='`.
         """
         dict = collections.defaultdict(list)
-        for name, value in self.getquerylist(delims, sep, encoding):
+        for name, value in self.getquerylist(delims, encoding):
             dict[name].append(value)
         return dict
 
@@ -475,15 +555,13 @@ class DefragResult(collections.namedtuple('DefragResult', 'base fragment')):
     """Class to hold :func:`uridefrag` results."""
 
     def geturi(self):
-        """Return the re-combined version of the original URI as a string."""
-        if self.fragment is not None:
-            try:
-                return self.base + '#' + self.fragment
-            except TypeError:
-                # Python 3: handle string as bytes
-                return self.base + b'#' + self.fragment
-        else:
+        """Return the recombined version of the original URI as a string."""
+        if self.fragment is None:
             return self.base
+        elif isinstance(self.fragment, bytes):
+            return b'#'.join((self.base, self.fragment))
+        else:
+            return '#'.join((self.base, self.fragment))
 
     def getbase(self, encoding='utf-8'):
         """Return the decoded absolute URI or relative URI reference without
@@ -497,6 +575,7 @@ class DefragResult(collections.namedtuple('DefragResult', 'base fragment')):
         original URI did not contain a fragment component.
 
         """
-        if self.fragment is None:
+        if self.fragment is not None:
+            return uridecode(self.fragment, encoding)
+        else:
             return default
-        return uridecode(self.fragment, encoding)
